/*
 * MIT License
 *
 * Copyright (c) 2019 norangebit
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

package it.norangeb.algorithms.graph.operations

import it.norangeb.algorithms.graph.DiGraph
import org.amshove.kluent.`should equal`
import org.spekframework.spek2.Spek
import org.spekframework.spek2.style.gherkin.Feature

object DepthFirstOrderTest : Spek({
    Feature("order of graph traverse") {
        Scenario("traverse linear graph") {
            val graph by memoized { DiGraph() }

            Given("graph 0 -> 1, 1 -> 2, 2 -> 3, 3 -> 4") {
                graph.addEdge(0, 1)
                graph.addEdge(1, 2)
                graph.addEdge(2, 3)
                graph.addEdge(3, 4)
            }

            lateinit var result: DepthFirstOrder

            When("execute a depth first order traver") {
                result = DepthFirstOrder(graph)
            }

            Then("preOrder should equal listOf(0, 1, 2, 3, 4)") {
                result.preOrder().toList() `should equal` listOf(0, 1, 2, 3, 4)
            }

            Then("postOrder should equal listOf(4, 3, 2, 1, 0)") {
                result.postOrder().toList() `should equal` listOf(4, 3, 2, 1, 0)
            }

            Then("reversePostOrder should equal listOf(0, 1, 2, 3, 4)") {
                result.reversePostOrder() `should equal` listOf(0, 1, 2, 3, 4)
            }
        }
    }
})